if [ -d var/lib/games/tmp ]; then
  ( cd var/lib/games/tmp
    for file in * ; do
      if [ ! -r ../$file ]; then
        mv $file ..
      fi
    done )
  rm -rf var/lib/games/tmp
fi
if [ -x usr/bin/scrollkeeper-update ]; then
  usr/bin/scrollkeeper-update -p var/lib/scrollkeeper 1> /dev/null 2> /dev/null
fi

if [ -x usr/bin/update-mime-database ]; then
  usr/bin/update-mime-database usr/share/mime 1> /dev/null 2> /dev/null
fi

if [ -x usr/bin/update-desktop-database ]; then
  usr/bin/update-desktop-database 1> /dev/null 2> /dev/null
fi
